#include "editor.h"
#include "input.h"
#include "image.h"
#include "screen.h"
#include "tilemap.h"
#include "timer.h"
#include "window.h"
#include "main.h"
#include "imagemap.h"
#include <iostream>
#include <fstream>

// static so we can access from a static method
bool Editor::prompt = false;
bool Editor::reallyQuit = false;

Editor::Editor () : filename("./levels/custom.lvl"), tileIndex(0), ix(0), iy(0) {
	tileSet = ImageMap::get("tiles");
	background = ImageMap::get("background");
	bool fileExists;
	tileMap = new TileMap(filename, fileExists);
	
	changed = ! fileExists;
	
	Image* yes = ImageMap::get("yes");
	Image* no = ImageMap::get("no");
	Image* play = ImageMap::get("play-small");
	Image* save = ImageMap::get("save");
	promptImage = ImageMap::get("prompt");
	
	quitButton = new Button(yes, 335, 310, quitAction);
	dontQuitButton = new Button(no, 415, 310, dontQuitAction);
	playButton = new Button(play, 32, 3, playAction);
	saveButton = new Button(save, 28, 3, saveAction);
	
	prompt = false;
}

Editor::~Editor () {
	delete tileMap;
	delete quitButton;
	delete dontQuitButton;
}

void Editor::handleInput (SDL_Event& event) {
	switch (event.type) {
		case SDL_KEYDOWN:
			keyDown(event);
		break;

		case SDL_MOUSEBUTTONDOWN:
			mouseDown(event);
		break;
		
		case SDL_QUIT:
			reallyQuit = true;
			promptSave();
		break;
	}
}

void Editor::keyDown (SDL_Event& event) {
	switch (event.key.keysym.sym) {
		case SDLK_ESCAPE:
			reallyQuit = false;
			promptSave();
		break;
		default: {}
	}
}

void Editor::mouseDown (SDL_Event& event) {
	switch (event.button.button) {
		case SDL_BUTTON_WHEELUP:
			// Scroll to next tile.
			tileIndex = (tileIndex + 1) % tileSet->getCount();
		break;

		case SDL_BUTTON_WHEELDOWN:
			// Scroll to previous tile.
			if (tileIndex != 0) {
				tileIndex--;
			} else {
				tileIndex = tileSet->getCount() - 1;
			}
		break;
	}
}

void Editor::saveToFile () {
	changed = false;
	
	std::ofstream file(filename);
	if (file.is_open()) {
		for (int y = 0; y < tileMap->HEIGHT; y++) {
			for (int x = 0; x < tileMap->WIDTH; x++) {
				file << tileMap->tileArray[x][y];
				if (x != tileMap->WIDTH - 1) {
					file << " ";
				}
			}
			file << "\n";
		}

		file.close();
	}
}

void Editor::update () {
	ix = Input::getMouseX() / 32;
	iy = Input::getMouseY() / 32;
	
	valid = true;
	
	if (ix < 1) valid = false;
	if (ix > 23) valid = false;
	if (iy < 1) valid = false;
	if (iy > 18) valid = false;

	if (prompt) {
		quitButton->update();
		dontQuitButton->update();
		return;
	}
	
	//playButton->update();
	
	if (changed) {
		saveButton->update();
	}
	
	if (! valid) return;
	
	if (Input::isMousePressed(1)) {
		tileMap->tileArray[ix][iy] = tileIndex;
		changed = true;
	} else if (Input::isMousePressed(3)) {
		tileMap->tileArray[ix][iy] = 0;
		changed = true;
	}
}

void Editor::draw () const {

	// Draw a background
	background->draw(0, 0);
	
	// Draw the contents of the tileArray to the screen.
	for (int i = 0; i < 25; i++) {
		for (int j = 0; j < 20; j++) {
			tileSet->drawFrame(tileMap->tileArray[i][j], i * 32, j * 32);
		}
	}

	if (prompt) {
		promptImage->draw(300, 250);
		quitButton->draw();
		dontQuitButton->draw();
	} else {
		if (valid && ! Input::isMousePressed(3)) {
			tileSet->drawFrame(tileIndex, ix * 32, iy * 32);
		}
	}
	
	//playButton->draw();
	
	if (changed) {
		saveButton->draw();
	}
	
	SDL_Flip(Window::getSurface());
}

void Editor::promptSave() {
	if (changed && ! prompt) {
		prompt = true;
	} else {
		quitAction(0);
	}	
}

void Editor::quitAction (Button* b) {
	prompt = false;
	if (reallyQuit) {
		exit(0);
	} else {
		transitionTo(titleScreen);
	}
}

void Editor::dontQuitAction (Button* b) {
	prompt = false;
}

void Editor::playAction (Button* b) {
	
	transitionTo(titleScreen);
}

void Editor::saveAction (Button* b) {
	static_cast<Editor*>(editor)->saveToFile();
}

