#include "player.h"
#include "imagemap.h"
#include "input.h"
#include "bullet.h"
#include "level.h"
#include "entity.h"
#include "tile.h"
#include "baby.h"

#include <math.h>
#include <iostream>

Player::Player (float x, float y) : MovingEntity(x, y), standing(false), fireDelay(0), weapon(0), batFrame(-1), previousBatFrame(0) {
	image = ImageMap::get("player");
	previousTile = 0;

	// Initialise the frame list for the walking animation.
	std::list<int> walkingFrames;
	walkingFrames.push_back(0);
	walkingFrames.push_back(1);
	walkingFrames.push_back(0);
	walkingFrames.push_back(2);
	walking = new Animation(15, walkingFrames);
}

void Player::update () {
	Level* level = Level::current();
	Entity* currentTile = level->findEntity(x + 16, y + 16, 0);

	int dir = Input::isPressed(SDLK_RIGHT) - Input::isPressed(SDLK_LEFT);
	
	updateX(dir * 3.0f);
	
	if (standing && Input::isPressed(SDLK_UP)) {
		const int jumpHeight = 80;
		const float jumpVel = -sqrt(2 * GRAVITY * jumpHeight);
		
		vy = jumpVel;
	}
	
	standing = false;
	
	updateY();
	
	// Check for weapon pickups.
	if (currentTile != 0 && currentTile != previousTile) {
		int type = currentTile->getType();
		if (type == Tile::BAT) {
			if (weapon == NOTHING) {
				weapon = BAT;
				Tile* tile = static_cast<Tile*>(currentTile);
				tile->type = Tile::BLANK;
			} else if (weapon == BAT) {
				// Do nothing for now?
			} else if (weapon == SLINGSHOT) {
				// Need to drop the slingshot here.
				Tile* tile = static_cast<Tile*>(currentTile);
				weapon = BAT;
				fireDelay = 0;
				tile->type = Tile::SLINGSHOT;
			}
		} else if (type == Tile::SLINGSHOT) {
			if (weapon == NOTHING) {
				weapon = SLINGSHOT;
				Tile* tile = static_cast<Tile*>(currentTile);
				tile->type = Tile::BLANK;
			} else if (weapon == BAT) {
				// Need to drop the bat here.
				Tile* tile = static_cast<Tile*>(currentTile);
				weapon = SLINGSHOT;
				fireDelay = 0;
				tile->type = Tile::BAT;
			} else if (weapon == SLINGSHOT) {
				// Do nothing for now?
			}
		}
	}

	// Use the weapon.
	if (Input::isPressed(SDLK_SPACE)) {
		switch(weapon) {
			case NOTHING:
				break;

			case BAT: {
				// Set batFrame to 0 to start the bat animation.
				if (batFrame == -1) {
					batFrame = SHOULDER;
					previousBatFrame = SHOULDER;
					fireDelay = 3;
				}
				break;
			}

			case SLINGSHOT:
				if (fireDelay <=0) {
					Bullet::create(x + 16, y + 16, facing);
					fireDelay = 40;
				}
				break;
		}	
	}

	// Update the bat animation.
	if (weapon == BAT && fireDelay == 0) {
		switch (batFrame) {
			case SHOULDER:
				if (previousBatFrame != UPRIGHT) {
					batFrame = UPRIGHT;
					previousBatFrame = SHOULDER;
					fireDelay = 2.5;
				} else {
					batFrame = -1;
				}
				break;

			case UPRIGHT:
				if (previousBatFrame == SHOULDER) {
					batFrame = HITTING;
					previousBatFrame = UPRIGHT;
					fireDelay = 3;
	
					// Check if you hit a baby.
					Entity* facingTile = level->findEntity(x + 16 + (16 * facing), y + 16, this);
					if (facingTile) {
						int type = facingTile->getType();
						if (type == Tile::BABY) {
							Baby* baby = static_cast<Baby*>(facingTile);
							baby->cry();
		
						}
					}
				} else {
					batFrame = SHOULDER;
					previousBatFrame = UPRIGHT;
					fireDelay = 3;
				}
				break;

			case HITTING:
				batFrame = UPRIGHT;
				previousBatFrame = HITTING;
				fireDelay = 2.5;
				break;
		}
	}

	if (fireDelay >= 1) {
		fireDelay--;
	}

	// Control the player's walking animation.
	if (dir != 0 && standing) {
	  walking->step();
	} else {
		walking->reset();
	}

	frame = walking->getFrame();
	
	previousTile = currentTile;
}

void Player::draw () const {
	MovingEntity::draw();

	if (weapon == BAT) {
		Image* bat_animation = ImageMap::get("bat_animation");
		int drawFrame = batFrame;
		if (drawFrame == -1) {
			drawFrame = 0;
		}
		if (facing == 1) {
			bat_animation->drawFrame(drawFrame, x - 16, y - 16);
		} else {
			bat_animation->drawFrame(drawFrame, x - 16, y - 16, true);
		}
	}
}

void Player::standingOn (Entity* entity) {
	if (vy >= 0) {
		standing = true;
		
		MovingEntity::standingOn(entity);
	}
}

void Player::bangingHead (Entity* entity) {
	if (vy < 0) {
		MovingEntity::bangingHead(entity);
		vy = 0;
	}
}

void Player::stoodOnBy (MovingEntity* entity, float speed) {
	if (speed >= 10.0f) {
		// die;
	}
}
